//
// Title:   SRFFileFormat.h
// Created: Rfi APR 18 09:00:00 2003
// Authors: Tamy Boubekeur
//
//
// This file is part of the CoTeX project.
// See http://www.cotex.fr.st/ for more information.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later
// version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General
// Public License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place, Suite 330,
// Boston, MA 02111-1307 USA
// 
// Contact tamyboubekeur@libertysurf.fr if any conditions of this
// licensing are not clear to you.
//

#ifndef __SRFFILEFORMAT_H_
#define __SRFFILEFORMAT_H_

#include <qlist.h>
#include <iostream>
#include <fstream>

#include "../../../Core/Interfaces/src/FileFormatPluginInterface.h"
#include "../../../Core/Scene/src/Object.h"
#include "../../../Core/DataStructures/src/SurfelCollection.h"
#include "../../../Core/MarkerManager/src/Marker3D.h"

/// SRF don't provide radius all the time
static const double DEFAULT_RADIUS = 1.0;

/**
 * This class implements loader/saver pointshop
 * plugin for the srf format.
 *
 * @author Tamy Boubekeur
 * @version 0.1
 */
class SRFFileFormat : public FileFormatPluginInterface {

public:
	
  FILEFORMATPLUGIN_API SRFFileFormat();
  FILEFORMATPLUGIN_API virtual ~SRFFileFormat();

  // *****************************
  // interface FileFormatInterface
  // *****************************

  /**
   * srf format is too simple-minded for having a 
   * good detection of bad/wrong files. (no header).
   */
  FILEFORMATPLUGIN_API bool isReadable (const FileHeader fileHeader);
  FILEFORMATPLUGIN_API bool readFromFile (const QString filePath, const bool mergeWithCurrent = false);
  FILEFORMATPLUGIN_API bool writeToFile (const QString filePath, const bool onlyActiveObject = false);
  FILEFORMATPLUGIN_API QString getFileFilter() const;
  FILEFORMATPLUGIN_API QString getFileSuffix() const;
  FILEFORMATPLUGIN_API void customize (QWidget *parent);

private:
  
  bool storeRadius;
  
  /// If there's no radius, return the DEFAULT_RADIUS
  float readHeader (std::ifstream & input);
  void writeObjectToStream (Object * object, std::ofstream & output);

  /// The average radius is defined on the active object.
  float defineAverageRadius (Object * o);

};

// ****************
// access functions
// ****************

// extern "C" is needed so those functions can be accessed
// with GetProcAddress() (WIN32) and dlsym() (UNIX) respective
extern "C" {
  FILEFORMATPLUGIN_API FileFormatPluginInterface *createFileFormatPlugin();
}

#endif  // __SRFFILEFORMAT_H_

// Some Emacs-Hints -- please don't remove:
//
//  Local Variables:
//  mode:C++
//  tab-width:4
//  End:
